import type { NodePath } from '@babel/core';
import * as t from '@babel/types';
import type { Metadata } from '../types';
/**
 * Takes `cssMap` function expression and then transforms it to a record of class names and sheets.
 *
 * For example:
 * ```
 * const styles = cssMap({
 *    none: { color: 'red' },
 *    solid: { color: 'green' },
 * });
 * ```
 * gets transformed to
 * ```
 * const styles = {
 *    danger: "_syaz5scu",
 *    success: "_syazbf54",
 * };
 * ```
 *
 * @param path {NodePath} The path to be evaluated.
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
export declare const visitCssMapPath: (path: NodePath<t.CallExpression> | NodePath<t.TaggedTemplateExpression>, meta: Metadata) => void;
